import { Injectable } from '@angular/core';
import { NatsService } from 'app/core/nats/services/nats.service';
import * as nats from 'nats.ws';

import { Error } from 'app/modules/protobuf/protobuf/rpc/error_pb';

import {
  ListMembersRequest,
  ListMembersResponse,
  GetMemberByUsernameRequest,
  GetMemberByUsernameResponse,
  GetMemberRequest,
  GetMemberResponse,
  SUBJECT_LIST_MEMBERS,
  SUBJECT_GET_MEMBER,
  SUBJECT_GET_MEMBER_BY_USERNAME,
} from 'app/modules/protobuf/c2se/backend/member_pb';

@Injectable({
  providedIn: 'root',
})
export class MembeerService {
  /**
   * Constructor
   */
  constructor(private __natsService: NatsService) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Accessors
  // -----------------------------------------------------------------------------------------------------

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  listMembers(): Promise<ListMembersResponse.Result> {
    return new Promise<ListMembersResponse.Result>((resolve, reject) => {
      let req = new ListMembersRequest();

      this.__natsService
        .request<ListMembersResponse.Result>(
          SUBJECT_LIST_MEMBERS,
          req.serializeBinary(),
          ListMembersResponse.deserializeBinary
        )
        .then((result) => {
          console.log('success', result, result.getMembersList());
          resolve(result);
        })
        .catch((e: Error) => {
          console.log('failed', e);
          reject(e);
        });
    });
  }

  getMember(id: string): Promise<GetMemberResponse.Result> {
    return new Promise<GetMemberResponse.Result>((resolve, reject) => {
      let req = new GetMemberRequest();
      req.setId(id);

      this.__natsService
        .request<GetMemberResponse.Result>(
          SUBJECT_GET_MEMBER,
          req.serializeBinary(),
          GetMemberResponse.deserializeBinary
        )
        .then((result) => {
          console.log('success', result, result.getMember());
          resolve(result);
        })
        .catch((e: Error) => {
          console.log('failed', e);
          reject(e);
        });
    });
  }

  getMemberByUsername(): Promise<GetMemberByUsernameResponse.Result> {
    return new Promise<GetMemberByUsernameResponse.Result>(
      (resolve, reject) => {
        let req = new GetMemberByUsernameRequest();

        this.__natsService
          .request<GetMemberByUsernameResponse.Result>(
            SUBJECT_GET_MEMBER_BY_USERNAME,
            req.serializeBinary(),
            GetMemberByUsernameResponse.deserializeBinary
          )
          .then((result) => {
            resolve(result);
          })
          .catch((e: Error) => {
            console.log('failed', e);
            reject(e);
          });
      }
    );
  }
}
